@@ -19,7 +19,14 @@
  *
  */
 
-#include "internal.h"
+#if HAVE_CONFIG_H
+# include "config.h"
+#endif
+
+#include <stdlib.h>
+#include <string.h>
+
+#include "stringprep.h"
 
 /* This file contains functions from GLIB, including gutf8.c and
  * gunidecomp.c, all licensed under LGPL and copyright hold by:
@@ -351,6 +358,8 @@ g_utf8_to_ucs4_fast (const gchar * str, glong len, glong * items_written)
     }
 
   result = g_new (gunichar, n_chars + 1);
+  if (!result)
+    return NULL;
 
   p = str;
   for (i = 0; i < n_chars; i++)
@@ -460,6 +469,8 @@ g_ucs4_to_utf8 (const gunichar * str,
     }
 
   result = g_malloc (result_length + 1);
+  if (!result)
+    return NULL;
   p = result;
 
   i = 0;
@@ -765,6 +776,8 @@ _g_utf8_normalize_wc (const gchar * str, gssize max_len, GNormalizeMode mode)
     }
 
   wc_buffer = g_new (gunichar, n_wc + 1);
+  if (!wc_buffer)
+    return NULL;
 
   last_start = 0;
   n_wc = 0;
