@@ -31,7 +31,7 @@ extern "C"
 #include <unistd.h>		/* ssize_t */
 #include <idn-int.h>		/* uint32_t */
 
-#define STRINGPREP_VERSION "0.3.2"
+#define STRINGPREP_VERSION "0.3.3"
 
 /* Error codes. */
   typedef enum
@@ -88,49 +88,41 @@ extern "C"
   {
     Stringprep_profile_steps operation;
     Stringprep_profile_flags flags;
-    Stringprep_table_element *table;
-    char *name;
+    const Stringprep_table_element *table;
   };
   typedef struct Stringprep_table Stringprep_profile;
 
   struct Stringprep_profiles
   {
     char *name;
-    Stringprep_profile *tables;
+    const Stringprep_profile *tables;
   };
   typedef struct Stringprep_profiles Stringprep_profiles;
 
-  extern Stringprep_profiles stringprep_profiles[];
+  extern const Stringprep_profiles stringprep_profiles[];
 
 /* Profiles */
-  extern Stringprep_table_element stringprep_rfc3454_A_1[];
-  extern Stringprep_table_element stringprep_rfc3454_B_1[];
-  extern Stringprep_table_element stringprep_rfc3454_B_2[];
-  extern Stringprep_table_element stringprep_rfc3454_B_3[];
-  extern Stringprep_table_element stringprep_rfc3454_C_1_1[];
-  extern Stringprep_table_element stringprep_rfc3454_C_1_2[];
-  extern Stringprep_table_element stringprep_rfc3454_C_2_1[];
-  extern Stringprep_table_element stringprep_rfc3454_C_2_2[];
-  extern Stringprep_table_element stringprep_rfc3454_C_3[];
-  extern Stringprep_table_element stringprep_rfc3454_C_4[];
-  extern Stringprep_table_element stringprep_rfc3454_C_5[];
-  extern Stringprep_table_element stringprep_rfc3454_C_6[];
-  extern Stringprep_table_element stringprep_rfc3454_C_7[];
-  extern Stringprep_table_element stringprep_rfc3454_C_8[];
-  extern Stringprep_table_element stringprep_rfc3454_C_9[];
-  extern Stringprep_table_element stringprep_rfc3454_D_1[];
-  extern Stringprep_table_element stringprep_rfc3454_D_2[];
-
-  /* Generic (for debugging) */
-
-  extern Stringprep_profile stringprep_generic[];
-
-#define stringprep_generic(in, maxlen)		\
-  stringprep(in, maxlen, 0, stringprep_generic)
+  extern const Stringprep_table_element stringprep_rfc3454_A_1[];
+  extern const Stringprep_table_element stringprep_rfc3454_B_1[];
+  extern const Stringprep_table_element stringprep_rfc3454_B_2[];
+  extern const Stringprep_table_element stringprep_rfc3454_B_3[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_1_1[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_1_2[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_2_1[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_2_2[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_3[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_4[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_5[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_6[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_7[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_8[];
+  extern const Stringprep_table_element stringprep_rfc3454_C_9[];
+  extern const Stringprep_table_element stringprep_rfc3454_D_1[];
+  extern const Stringprep_table_element stringprep_rfc3454_D_2[];
 
   /* Nameprep */
 
-  extern Stringprep_profile stringprep_nameprep[];
+  extern const Stringprep_profile stringprep_nameprep[];
 
 #define stringprep_nameprep(in, maxlen)			\
   stringprep(in, maxlen, 0, stringprep_nameprep)
@@ -140,25 +132,25 @@ extern "C"
 
   /* SASL */
 
-  extern Stringprep_profile stringprep_saslprep[];
-  extern Stringprep_profile stringprep_plain[];
-  extern Stringprep_profile stringprep_trace[];
+  extern const Stringprep_profile stringprep_saslprep[];
+  extern const Stringprep_profile stringprep_plain[];
+  extern const Stringprep_profile stringprep_trace[];
 
 #define stringprep_plain(in, maxlen)		\
   stringprep(in, maxlen, 0, stringprep_plain)
 
   /* Kerberos */
 
-  extern Stringprep_profile stringprep_kerberos5[];
+  extern const Stringprep_profile stringprep_kerberos5[];
 
 #define stringprep_kerberos5(in, maxlen)		\
   stringprep(in, maxlen, 0, stringprep_kerberos5)
 
   /* XMPP */
 
-  extern Stringprep_profile stringprep_xmpp_nodeprep[];
-  extern Stringprep_profile stringprep_xmpp_resourceprep[];
-  extern Stringprep_table_element stringprep_xmpp_nodeprep_prohibit[];
+  extern const Stringprep_profile stringprep_xmpp_nodeprep[];
+  extern const Stringprep_profile stringprep_xmpp_resourceprep[];
+  extern const Stringprep_table_element stringprep_xmpp_nodeprep_prohibit[];
 
 #define stringprep_xmpp_nodeprep(in, maxlen)		\
   stringprep(in, maxlen, 0, stringprep_xmpp_nodeprep)
@@ -167,20 +159,26 @@ extern "C"
 
   /* iSCSI */
 
-  extern Stringprep_profile stringprep_iscsi[];
+  extern const Stringprep_profile stringprep_iscsi[];
 
 #define stringprep_iscsi(in, maxlen)		\
   stringprep(in, maxlen, 0, stringprep_iscsi)
 
   /* API */
 
+  extern int stringprep_4i (uint32_t * ucs4, size_t * len, size_t maxucs4len,
+			    Stringprep_profile_flags flags,
+			    const Stringprep_profile * profile);
+  extern int stringprep_4zi (uint32_t * ucs4, size_t maxucs4len,
+			     Stringprep_profile_flags flags,
+			     const Stringprep_profile * profile);
   extern int stringprep (char *in, size_t maxlen,
 			 Stringprep_profile_flags flags,
-			 Stringprep_profile * profile);
+			 const Stringprep_profile * profile);
 
   extern int stringprep_profile (const char *in,
 				 char **out,
-				 char *profile,
+				 const char *profile,
 				 Stringprep_profile_flags flags);
 
   extern const char *stringprep_check_version (const char *req_version);
